@@ -7,13 +7,14 @@ use Class::Struct;
 
 use Verilog::Netlist;
 use Verilog::Netlist::Net;
+use Verilog::Netlist::Pin;
 use Verilog::Netlist::Subclass;
 use vars qw($VERSION @ISA);
 use strict;
 @ISA = qw(Verilog::Netlist::Interface::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '3.202';
+$VERSION = '3.210';
 
 structs('new',
 	'Verilog::Netlist::Interface::Struct'
@@ -25,7 +26,11 @@ structs('new',
 	   attributes	=> '%', #'	# Misc attributes for systemperl or other processors
 	   #
 	   comment	=> '$', #'	# Comment provided by user
+	   _ports	=> '%',		# hash of Verilog::Netlist::Ports
+	   _portsordered=> '@',		# list of Verilog::Netlist::Ports as ordered in list of ports
 	   _nets	=> '%',		# hash of Verilog::Netlist::Nets
+	   _cells	=> '%',		# hash of Verilog::Netlist::Cells
+	   _level	=> '$',		# Depth in hierarchy (if calculated)
 	   ]);
 
 sub delete {
@@ -33,6 +38,12 @@ sub delete {
     foreach my $oref ($self->nets) {
 	$oref->delete;
     }
+    foreach my $oref ($self->ports) {
+	$oref->delete;
+    }
+    foreach my $oref ($self->cells) {
+	$oref->delete;
+    }
     my $h = $self->netlist->{_interfaces};
     delete $h->{$self->name};
     return undef;
@@ -42,10 +53,30 @@ sub delete {
 
 sub is_top {}  # Ignored, for module compatibility
 
+sub keyword { return 'interface'; }
+
 sub logger {
     return $_[0]->netlist->logger;
 }
 
+sub find_port {
+    my $self = shift;
+    my $search = shift;
+    return $self->_ports->{$search} || $self->_ports->{"\\".$search." "};
+}
+sub find_port_by_index {
+    my $self = shift;
+    my $myindex = shift;
+    # @{$self->_portsordered}[$myindex-1] returns the name of
+    # the port in the module at this index.  Then, this is
+    # used to find the port reference via the port hash
+    return $self->_ports->{@{$self->_portsordered}[$myindex-1]};
+}
+sub find_cell {
+    my $self = shift;
+    my $search = shift;
+    return $self->_cells->{$search} || $self->_cells->{"\\".$search." "};
+}
 sub find_net {
     my $self = shift;
     my $search = shift;
@@ -63,18 +94,33 @@ sub nets {
 sub nets_sorted {
     return (sort {$a->name() cmp $b->name()} (values %{$_[0]->_nets}));
 }
-sub nets_and_ports_sorted {
+sub ports {
+    return (values %{$_[0]->_ports});
+}
+sub ports_sorted {
+    return (sort {$a->name() cmp $b->name()} (values %{$_[0]->_ports}));
+}
+sub ports_ordered {
     my $self = shift;
-    return $self->nets_sorted(@_);
+    return map {$self->_ports->{$_}} @{$self->_portsordered};
+}
+sub cells {
+    return (values %{$_[0]->_cells});
+}
+sub cells_sorted {
+    return (sort {$a->name() cmp $b->name()} (values %{$_[0]->_cells}));
+}
+sub nets_and_ports_sorted {
+    return Verilog::Netlist::Module::nets_and_ports_sorted(@_);
 }
 
 sub new_net {
     my $self = shift;
     # @_ params
-    # Create a new net under this interface
+    # Create a new net under this
     my $netref = new Verilog::Netlist::Net (direction=>'net', data_type=>'wire',
 					    @_,
-					    interface=>$self, );
+					    module=>$self, );
     $self->_nets ($netref->name(), $netref);
     return $netref;
 }
@@ -85,31 +131,83 @@ sub new_attr {
     push @{$self->attrs}, $clean_text;
 }
 
+sub new_port {
+    my $self = shift;
+    # @_ params
+    # Create a new port under this module
+    my $portref = new Verilog::Netlist::Port (@_, module=>$self,);
+    $self->_ports ($portref->name(), $portref);
+    return $portref;
+}
+
+sub new_cell {
+    return Verilog::Netlist::Module::new_cell(@_);
+}
+
+sub level {
+    my $self = shift;
+    my $level = $self->_level;
+    return $level if defined $level;
+    $self->_level(2);  # Interfaces are never up "top"
+    foreach my $cell ($self->cells) {
+	if ($cell->submod) {
+	    my $celllevel = $cell->submod->level;
+	    $self->_level($celllevel+1) if $celllevel >= $self->_level;
+	}
+    }
+    return $self->_level;
+}
+
 sub link {
     my $self = shift;
     # Ports create nets, so link ports before nets
+    foreach my $portref ($self->ports) {
+	$portref->_link();
+    }
     foreach my $netref ($self->nets) {
 	$netref->_link();
     }
+    foreach my $cellref ($self->cells) {
+	$cellref->_link();
+    }
 }
 
 sub lint {
     my $self = shift;
     if (!$self->netlist->{skip_pin_interconnect}) {
+	foreach my $portref ($self->ports) {
+	    $portref->lint();
+	}
 	foreach my $netref ($self->nets) {
 	    $netref->lint();
 	}
     }
+    foreach my $cellref ($self->cells) {
+	$cellref->lint();
+    }
 }
 
 sub verilog_text {
     my $self = shift;
     my @out = "interface ".$self->name." (\n";
     my $indent = "   ";
+    # Port list
+    my $comma="";
+    push @out, $indent;
+    foreach my $portref ($self->ports_sorted) {
+	push @out, $comma, $portref->verilog_text;
+	$comma = ", ";
+    }
+    push @out, ");\n";
+
     # Signal list
     foreach my $netref ($self->nets_sorted) {
 	push @out, $indent, $netref->verilog_text, "\n";
     }
+    # Cell list
+    foreach my $cellref ($self->cells_sorted) {
+	push @out, $indent, $cellref->verilog_text, "\n";
+    }
 
     push @out, "endinterface\n";
     return (wantarray ? @out : join('',@out));
@@ -121,9 +219,15 @@ sub dump {
     my $norecurse = shift;
     print " "x$indent,"Interface:",$self->name(),"  File:",$self->filename(),"\n";
     if (!$norecurse) {
+	foreach my $portref ($self->ports_sorted) {
+	    $portref->dump($indent+2);
+	}
 	foreach my $netref ($self->nets_sorted) {
 	    $netref->dump($indent+2);
 	}
+	foreach my $cellref ($self->cells_sorted) {
+	    $cellref->dump($indent+2);
+	}
     }
 }
 
@@ -189,6 +293,20 @@ Returns list of name sorted references to Verilog::Netlist::Net in the interface
 Returns list of name sorted references to Verilog::Netlist::Net and
 Verilog::Netlist::Port in the interface.
 
+=item $self->ports
+
+Returns list of references to Verilog::Netlist::Port in the module.
+
+=item $self->ports_ordered
+
+Returns list of references to Verilog::Netlist::Port in the module sorted
+by pin number.
+
+=item $self->ports_sorted
+
+Returns list of references to Verilog::Netlist::Port in the module sorted
+by name.
+
 =back
 
 =head1 MEMBER FUNCTIONS
@@ -205,6 +323,11 @@ Updates the AUTOs for the interface.
 
 Returns Verilog::Netlist::Net matching given name.
 
+=item $self->level
+
+Returns the reverse depth of this interface with respect to other modules
+and interfaces.  See also Netlist's modules_sorted_level.
+
 =item $self->lint
 
 Checks the interface for errors.
