@@ -105,26 +105,34 @@ private:
     VAstEnt() { assert(0); }	// Not made by users, it's an AV*
     ~VAstEnt() { assert(0); }	// Not made by users, it's an AV*
 
-    static av* newAVEnt (VAstType type);
-    static void initAVEnt (struct av* avp, VAstType type);
+    av* newAVEnt (VAstType type);
+    static void initAVEnt (struct av* avp, VAstType type, struct av* parentp);
 
     // ACCESSORS
     inline struct av* castAVp() { return (struct av*)(this); }
     inline VAstEnt* avToSymEnt(struct av* avp) { return (VAstEnt*)(avp); }
 
-    /// $self->[1]: For current entry, the hash of symbols under it
+    /// $self->[2]: For current entry, the hash of symbols under it
     struct hv* subhash();
 
     /// Insert into current table
     void insert(VAstEnt* newentp, const string& name);
 
 public:
+    // ACCESSORS
+
     /// $self->[0]: For current entry, the node type
     VAstType type();
 
+    /// $self->[1]: For current entry, what node it is under or NULL if netlist
+    VAstEnt* parentp();
+
     /// type() indicates we shouldn't report this as a containing object
     bool typeIgnoreObjof() { VAstType t=type(); return t==VAstType::BLOCK || t==VAstType::FORK; }
 
+    /// Info on current node, for debug
+    string ascii(const string& name="");
+
     // METHODS
     /// Return internal pointer for given name or null
     VAstEnt* findSym (const string& name);
